require_relative('die')
require_relative('game_turn')
require_relative('treasure_trove')
require 'csv'
module StudioGame
  class Game
      attr_reader :title

      def initialize(title)
          @title = title
          @players = []
      end

      def add_player(player)
          @players << player
      end

      def high_score_entry(player)
        "#{player.name.ljust(30, '.')} #{player.score}"
      end
=begin
      def load_players(from_file)
        File.readlines(from_file).each do |line|
          add_player(Player.from_csv(line))
        end
      end
=end
      def load_players(from_file)
        CSV.foreach(from_file) do |row|
          add_player(Player.new(row.first, Integer(row.last)))
        end
      end

      def save_high_scores(to_file='high_scores.txt')
        File.open(to_file,'w') do |file|
          file.puts "#{@title} High Scores:\n"
          @players.sort.each do |player|
            file.puts high_score_entry(player)
          end
        end
      end

      def play(rounds)
          puts "There are #{@players.size} players in #{@title}:"
          puts @players
          treasures = TreasureTrove::TREASURES
          puts "\nThe trove contains #{treasures.size} treasures!"
          treasures.each do |tr|
              puts "#{tr.name.to_s.ljust(30, '_')} #{tr.points}"
          end
          1.upto(rounds) do |round|
              if block_given?
                break if yield
              end
              puts "\nRound #{round}:"
              @players.each do |a_player|
                  GameTurn.take_turn(a_player)
                  puts a_player
              end
          end
      end

      def print_name_and_health(players)
          players.each { |player| puts "#{player.name} (#{player.health})" }
      end

      def total_points
          @players.reduce(0) { |sum, player| sum + player.points }
      end

      def print_stats
          strong, wimpy = @players.partition(&:strong?)
          puts "\n#{@title} Statistics:"
          puts "\n#{strong.size} strong players:"
          print_name_and_health(strong)
          puts "\n#{wimpy.size} wimpy players:"
          print_name_and_health(wimpy)
          puts "\n#{@title} High Scores:"
          @players.sort.each { |player| puts high_score_entry(player) }
          puts "\n#{@title} Total points:"
          @players.sort.each do |player|
              puts "\n#{player.name}'s point totals:"
              player.each_found_treasure do |tr|
                  puts "#{tr.points} total #{tr.name} points"
              end
              puts "#{player.points} grand total points"
          end
          puts "\n#{@title} Grand Total points:"
          puts total_points.to_s
      end
  end
end
